// This is -*- C++ -*-

/* 
 * types.h
 *
 * Copyright 1998 Karl E. Nelson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
#ifndef _GDKMM_TYPES_H_
#define _GDKMM_TYPES_H_

#include "gdk/gdk.h"
#include "gdkmmconfig.h"

#include "glibmm/ustring.h"

// we need memset, memcpy
#include <string.h>

/*********************************************************************
***** Version macros
*********************************************************************/
/* we need features to get the gdk version numbers. */
//#include <gtk/gtkfeatures.h>
//#include <gtk/gtkcompat.h>

/* macro for controlling version numbers */
#ifndef _GDK_VERSION
// this one is old.  Will be replaced with more flexable below. TODO: What exactly is old? Delete old stuff.
#define _GDK_VERSION(major,minor) ((GTK_MAJOR_VERSION==major)&&(GTK_MINOR_VERSION==minor))

#define GDK_VERSION_GT(major,minor) ((GTK_MAJOR_VERSION>major)||(GTK_MAJOR_VERSION==major)&&(GTK_MINOR_VERSION>minor))
#define GDK_VERSION_GE(major,minor) ((GTK_MAJOR_VERSION>major)||(GTK_MAJOR_VERSION==major)&&(GTK_MINOR_VERSION>=minor))
#define GDK_VERSION_EQ(major,minor) ((GTK_MAJOR_VERSION==major)&&(GTK_MINOR_VERSION==minor))
#define GDK_VERSION_NE(major,minor) ((GTK_MAJOR_VERSION!=major)||(GTK_MINOR_VERSION!=minor))
#define GDK_VERSION_LE(major,minor) ((GTK_MAJOR_VERSION<major)||(GTK_MAJOR_VERSION==major)&&(GTK_MINOR_VERSION<=minor))
#define GDK_VERSION_LT(major,minor) ((GTK_MAJOR_VERSION<major)||(GTK_MAJOR_VERSION==major)&&(GTK_MINOR_VERSION<minor))

//Note: Use GTK_CHECK_VERSION to check that a version is equal or more than (the micro version).

#endif

namespace Gdk {

/*********************************************************************
***** Forward declarations
*********************************************************************/


class Drawable;
class Bitmap;
class Colormap;
class Image;
class Pixmap;
class Window;
class Font;
class GC;
class Visual;


/* types */
typedef GdkAtom 		Atom;


/* Stucts */
class Color;
class GCValues;
class Geometry;



class Image; 
class Point;
class Points;
class Rectangle;
class Segment;
class Segments;
class WindowAttr;

class DeviceKey;

class TimeCoord;


/* Events - We need a better (light) wrapper, so unwrapped for now */
//typedef GdkEventAny		EventAny;
//typedef GdkEventExpose 	EventExpose;
//typedef GdkEventNoExpose	EventNoExpose;
//typedef GdkEventVisibility	EventVisibility;
//typedef GdkEventMotion	EventMotion;
//typedef GdkEventButton	EventButton;
//typedef GdkEventKey		EventKey;
//typedef GdkEventFocus		EventFocus;
//typedef GdkEventCrossing	EventCrossing;
//typedef GdkEventConfigure	EventConfigure;
//typedef GdkEventProperty	EventProperty;
//typedef GdkEventSelection	EventSelection;
//typedef GdkEventProximity	EventProximity;
//#if GDK_VERSION_LE(1,0)
//typedef GdkEventOther		EventOther;
//typedef GdkEventDragBegin	EventDragBegin;
//typedef GdkEventDragRequest	EventDragRequest;
//typedef GdkEventDropEnter	EventDropEnter;
//typedef GdkEventDropDataAvailable EventDropDataAvailable;
//typedef GdkEventDropLeave	EventDropLeave;
//#else
//typedef GdkEventDND           EventDND;
//#endif
//typedef GdkEventClient	EventClient;
//typedef GdkEvent 		Event;



/* Enums - We need a better encapsulation for Enums*/
//typedef GdkWindowType			WindowType;
//typedef GdkWindowClass		WindowClass;
//typedef GdkImageType			ImageType;
//typedef GdkVisualType			VisualType;
//typedef GdkFontType			FontType;
//typedef GdkWindowAttributesType	WindowAttributesType;
//typedef GdkWindowHints		WindowHints;
//typedef GdkFunction			Function;
//typedef GdkFill			Fill;
//typedef GdkFillRule			FillRule;
//typedef GdkLineStyle			LineStyle;
//typedef GdkCapStyle			CapStyle;
//typedef GdkJoinStyle			JoinStyle;
//typedef GdkCursorType			CursorType;
//typedef GdkFilterReturn		FilterReturn;
//typedef GdkVisibilityState		VisibilityState;
//typedef GdkEventType			EventType;
//typedef GdkEventMask			EventMask;
//typedef GdkNotifyType			NotifyType;
//typedef GdkModifierType		ModifierType;
//typedef GdkSubwindowMode		SubwindowMode;
//typedef GdkInputCondition		InputCondition;
//typedef GdkStatus			Status;
//typedef GdkByteOrder			ByteOrder;
//typedef GdkGCValuesMask		GCValuesMask;
//typedef GdkSelection			Selection;
//typedef GdkPropertyState		PropertyState;
//typedef GdkPropMode			PropMode;
//#if GDK_VERSION_LE(1,0)
//typedef GdkDndType			DndType;
//#endif
//typedef GdkInputSource		InputSource;
//typedef GdkInputMode			InputMode;
//typedef GdkAxisUse			AxisUse;
//typedef GdkTarget			Target;
//typedef GdkSelectionType		SelectionType;
//typedef GdkExtensionMode		ExtensionMode;
//typedef GdkIMStyle			IMStyle;
//typedef GdkWMDecoration		WMDecoration;
//typedef GdkWMFunction			WM_Function;
//typedef GdkColorContextMode		ColorContextMode;
//typedef GdkOverlapType		OverlapType;


/*********************************************************************
***** Implementations 
*********************************************************************/
/* Templates */

//: Object used as base to wrap all of the gdk structures
//- Gdk passes everything as pointers to structures.  To encapsulate
//- this and protect against bad pointers, we will wrap them in
//- a class with value semantics, but with a pointer access.
//- It can be directly changed back to the gdk equivalent (a pointer).
//- It is considered a private template.

template <class GdkObj>
  class Obj
    {
     public:
       typedef GdkObj BaseObjectType;
     protected:
       bool null_;
       GdkObj obj_;

       //: Initialize and zero memory
       Obj():null_(0)
         {
         }

     public:
       Obj(GdkObj *obj):null_(obj==NULL)
         {
          if (null_) 
            return;
          if (&obj_!=obj)
            memcpy(&obj_,obj,sizeof(GdkObj));
         }

       Obj(const Obj& obj):null_(0)
         {
          if (this!=&obj)
            memcpy(&obj_,&obj.obj_,sizeof(GdkObj));
         }

       ~Obj() {};

       operator GdkObj* ()
         {
          if (null_) 
            return 0;
          return &obj_;
         }

       operator GdkObj* () const
         {
          if (null_) 
            return 0;
          return &obj_;
         }

       GdkObj* gobj()
         {return &obj_;}
       const GdkObj* gobj() const
         {return &obj_;}

       GdkObj* operator -> ()
         {return &obj_;}
       const GdkObj* operator -> () const
         {return &obj_;}
    
    };



/* simple wrappers */

//class ColorContextDither	:public Obj<GdkColorContextDither> {};
//class ColorContext		:public Obj<GdkColorContext> {};
class GCValues		:public Obj<GdkGCValues> {};

class Geometry		:public Obj<GdkGeometry> {};
//class ICAttr		:public Obj<GdkICAttr> {};

class DragContext		:public Obj<GdkDragContext> 
{
public:
  DragContext () {};
  DragContext ( DragContext &x ) : 
    Obj<GdkDragContext> ( x ) {}
  DragContext ( GdkDragContext *x ):
    Obj<GdkDragContext> ( x ) {}
};


class WindowAttr		:public Obj<GdkWindowAttr> {};
class DeviceKey		:public Obj<GdkDeviceKey> {};
//class DeviceInfo		:public Obj<GdkDeviceInfo> {};
class TimeCoord		:public Obj<GdkTimeCoord> {};

/* Color */
//#include <gdkmm/color.h>

class Point: public Obj<GdkPoint>
  {
   public:
       // for convenience
       operator GdkPoint () const {
	   return obj_;
       }

     Point()
       :Obj<GdkPoint>() {}
     Point(GdkPoint *pt)
       :Obj<GdkPoint>(pt) {}
     Point(const Point& pt)
       :Obj<GdkPoint>(pt) {}
     Point(gint x,gint y)
       {obj_.x=x;
        obj_.y=y;
       }
     ~Point();
  };

class Points 
  {
    protected:
      GdkPoint *points_;
      int       size_;
      bool      owned_;

      // block assignment
      Points& operator= (const Points&);

      void a_alloc(size_t size);
      template <class Iterator>
      void a_dup(Iterator b, Iterator e) ;

    public:
      operator GdkPoint *() const { return points_; }
      int size() const            { return size_; }

      Points(GdkPoint * points, int size);
      Points(Point * points, int size);
      ~Points();

#ifndef GTKMM_CXX_AMBIGUOUS_TEMPLATES
      template <class Container>
      Points(const Container & c) 
        {
          a_dup(c.begin(), c.end());
        }
#else
      Points(const vector<Point> & c) { a_dup(c.begin(), c.end()); }
      Points(const list<Point> & c) { a_dup(c.begin(), c.end()); }
      Points(const vector<GdkPoint> & c) { a_dup(c.begin(), c.end()); }
      Points(const list<GdkPoint> & c) { a_dup(c.begin(), c.end()); }
#endif
      template <class Iterator>
      Points(Iterator b,Iterator e) { a_dup(b, e); }
  };

template <class Iterator>
void Points::a_dup(Iterator b,Iterator e)
  {
    Iterator iter;

    for(iter = b, size_ = 0; iter != e; ++iter, ++size_);

    a_alloc(size_);

    unsigned int i;
    for(iter = b, i = 0; iter != e; ++iter, ++i)
        points_[i] = *iter;
  }


class Rectangle : public Obj<GdkRectangle>
  {
   public:
     Rectangle()
       :Obj<GdkRectangle>() {}
     Rectangle(GdkRectangle* rect)
       :Obj<GdkRectangle>(rect) {}
     Rectangle(const Rectangle& rect)
       :Obj<GdkRectangle>(rect) {}
     Rectangle(gint x, gint y, gint w, gint h)
       :Obj<GdkRectangle>()
       {
        obj_.x = x;
        obj_.y = y;
        obj_.width = w;
        obj_.height = h;
       }
     ~Rectangle();

#if GDK_VERSION_GT(1,0)
     GdkRectangle join(Rectangle &src2)
       {GdkRectangle rec;
        gdk_rectangle_union(&obj_,&(src2.obj_),&rec);
        return rec;
       }
#endif
  };

#if GDK_VERSION_GT(1,0)
inline GdkRectangle join(Rectangle &src1,Rectangle &src2)
   {return src1.join(src2);
   }
#endif

class Segment: public Obj<GdkSegment>
  {
   public:
     Segment()
       :Obj<GdkSegment>() {}
     Segment(GdkSegment* seg)
       :Obj<GdkSegment>(seg) {}
     Segment(const Segment& seg)
       :Obj<GdkSegment>(seg) {}
     Segment(gint x1, gint y1, gint x2, gint y2)
       :Obj<GdkSegment>()
       {
        obj_.x1 = x1;
        obj_.y1 = y1;
        obj_.x2 = x2;
        obj_.y2 = y2;
       }
     ~Segment();
  };   

//: Remove a const and make gdk+.
//- C++ can't do a const_cast and use the converter to gdk
//- at the same time.  Therefore, we must show it the way.
template <class GdkObj>
typename GdkObj::BaseObjectType* gdk_const_cast(const GdkObj& obj)
{
  return const_cast<typename GdkObj::BaseObjectType*>( obj.gobj() );
}


} /* namespace Gdk */

#endif // _GDKMM_TYPES_H_

