// -*- c++ -*-
#ifndef _GLIBMM_OBJECT_H
#define _GLIBMM_OBJECT_H
/* $Id: object.h,v 1.23 2001/10/25 13:54:30 daniel Exp $ */

/* object.h
 * 
 * Copyright 2001 Free Software Foundation
 * Copyright 1998-1999 The gtkmm Development Team 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* this file is critical for our compile times, no #includes to this
   header, please */

//#include "gtkmm/base.h" //TODO: Just some #defines - this should probably be put somewhere else.
//#include <gobject/gobject.h>
#include "sigc++/object.h"

#include <glibmm/wrap.h>
#include <glibmm/containers.h>
#include <glibmm/quark.h>
#include <glibmm/proxy.h>
#include <glibmm/refptr.h>
#include "glibmm/utility.h" //Could be private, but that would be tedious.

typedef struct _GObject GObject;
typedef struct _GObjectClass GObjectClass;

typedef unsigned int GType;

namespace Glib
{

struct BadType
{
};

template <class T>
void delete_typed(void* data)
{
   delete (T*)(data);
}

class Object_Class;

//: The base class of the whole hierarchy
class Object : public SigC::Object
{
public:
  typedef Object       CppObjectType;
  typedef Object_Class CppClassType;
  typedef GObject      BaseObjectType;
  typedef GObjectClass BaseClassType;

  friend class GSigConnection_; //for GSigConnection_::disconnect().

protected:
  explicit Object(GObject* castitem = 0);
  virtual ~Object(); //It should only be deleted by the callback.

public:
  static RefPtr<Object> create(); //You must reimplement this in each derived class.

  virtual void reference() const; //Also used by libsigc++.
  virtual void unreference() const; //Also used by libsigc++.

  typedef void (*DestroyNotify) (gpointer data);

  static GType get_type();
  GObject* gobj()             { return gobject_; }
  const GObject* gobj() const { return gobject_; }

  GObject* gobj_copy(); //Give a ref-ed copy to someone. Use for direct struct access.


	
  GType type(); //TODO: Do we need this as well as get_type()?

  // Glib::Objects contain a list<Quark, pair<void*, DestroyNotify> >
  // to store run time data added to the object at run time.
  void* get_data(const QueryQuark &key);
  void set_data(const Quark &key, void* data);
  void set_data(const Quark &key, void* data, DestroyNotify notify);
  void remove_data(const QueryQuark& quark);
  // same as remove without notifying
  void* steal_data(const QueryQuark& quark);

  // convenience functions
  template <class T>
  void set_data_typed(const Quark& quark, const T& data)
    { set_data(quark, new T(data), delete_typed<T>); }

  template <class T>
  T& get_data_typed(const QueryQuark& quark)
    { return *static_cast<T*>(get_data(quark)); }

private:
  friend class CppClassType;
  static CppClassType object_class;

  Object(const Object&);
  Object& operator=(const Object&); // not implemented

  // Glib::Object can not be dynamic because it lacks a float state.
  virtual void set_manage();

protected:
  static void destroy_notify_callback_(gpointer data);
  virtual void destroy_notify_();

  GObject* gobject_; //The glib/gdk/gtk object instance.
  bool cpp_destruction_in_progress_;
};

} /* namespace Glib */

#endif /* _GLIBMM_OBJECT_H */

