// -*- c++ -*-
/* $Id: ustring.cc,v 1.3 2001/10/16 20:26:11 murrayc Exp $ */

/* ustring.h
 *
 * Copyright (C) 2001 The Gtk-- Development Team
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "glibmm/ustring.h"

namespace Glib
{


ustring::ustring()
{
}

ustring::ustring(const char* src)
{
  string_.assign(src);
}

ustring::ustring(const char* src, size_type chars_count)
{
  const char* end_=g_utf8_offset_to_pointer(src,chars_count);
  if (!end_)
    string_.assign(src);
  else
    string_.assign(src,end_);
}

ustring::ustring(const ustring& src)
{
  string_.assign(src.string_);
}

ustring::ustring(const std::string& src)
{
  string_.assign(src);
}

ustring& ustring::operator=(const ustring& src)
{
  string_ = src.string_;
  return *this;
}

ustring& ustring::operator=(const char* src)
{
  if(src)
    string_ = src;
  else
    string_.assign("");
  return *this;
}

ustring& ustring::operator=(const std::string& src)
{
  string_ = src;
  return *this;
}

ustring::~ustring()
{
}

ustring::operator std::string() const
{
  return string_;
}

bool ustring::operator==(const ustring& src) const
{
  return string_ == src.string_;
}

void ustring::operator+(const ustring& src)
{
  string_ += src.string_;
}

void ustring::operator+(const char* src)
{
  string_ += src;
}

ustring& ustring::operator+=(const ustring& src)
{
  string_ += src.string_;
  return *this;
}

ustring& ustring::operator+=(const char* src)
{
  string_ += src;
  return *this;
}

gunichar ustring::operator[](size_type i) const
{
  const gchar* pch = g_utf8_offset_to_pointer(string_.c_str(), i);
  return g_utf8_get_char(pch);
}

ustring ustring::substr(size_type pos, size_type n) const
{
  if(n == npos)
    n = size() - pos;

  const gchar* pch = g_utf8_offset_to_pointer(string_.c_str(), pos);
	return ustring(pch, n);
}

const char* ustring::c_str() const
{
  return string_.c_str();
}

ustring::size_type ustring::size(size_type ustring_max_length /*= 256 */) const
{
  return g_utf8_strlen(string_.c_str(), ustring_max_length);
}

bool ustring::empty() const
{
  return string_.size() == 0;
}

void ustring::clear()
{
  string_.assign("");
}

void ustring::erase()
{
  string_.erase();
}

} //namespace Glib.

