// -*- c++ -*-
#ifndef _GLIBMM_USTRING_H
#define _GLIBMM_USTRING_H

/* $Id: ustring.h,v 1.5 2001/10/19 08:30:49 murrayc Exp $ */

/* ustring.h
 *
 * Copyright (C) 2001 The Gtk-- Development Team
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <glib.h>
#include <string>

#include <glibmmconfig.h>
GTKMM_USING_STD_STRING


namespace Glib
{

//glib::ustring has much the same interface as std::string,
//but contains UTF8 strings.
//Note that a normal ANSI string is also a UTF8 string, so,
//if you want to, you can use this class without every thinking about UTF8.
//
//Also, this class has conversions to and from std::string,
//so you can use a std::string instead of a glib::ustring -
//However, this will not work with multi-byte translations,
//just as normal C char* code wouldn't.
//
//In a perfect world the C++ Standard Library would contain a
//a UTF8 string, but it doesn't.
//
//FIXME: URL of a simple description of UTF8.
//
//glib::ustring does not inherit from std::string, because std::string
//was intended to be a final class. For instance, it does not have
//a virtual destructor.
//

class ustring
{
public:
  typedef size_t size_type;
  static const size_type npos = static_cast<size_type>(-1);

  ustring();
  ustring(const ustring& src);
  ustring(const std::string& src);
  ustring(const char* pchSrc);
  ustring(const char* pchSrc, size_type bytes_count);
  ~ustring();

  ustring& operator=(const ustring& src);
  ustring& operator=(const char* pchSrc);
  ustring& operator=(const std::string& src);

  operator std::string() const; //e.g. std::string str = ustring;

  bool operator==(const ustring& src) const;

  gunichar ustring::operator[](size_type i) const; //Can't be used to change the value.

  void operator+(const ustring& src);
  void operator+(const char* src);

  ustring& operator+=(const ustring& src);
  ustring& operator+=(const char* src);

  size_type size(size_type ustring_max_length = npos) const;
  bool empty() const;

  ustring substr(size_type pos, size_type n=npos) const;
  void clear();

  const char* c_str() const; //Not necessarily an ANSI char*. Use g_utf8_*() where necessary.

  void erase();

protected:
  std::string string_;

};

}; // namespace Glib

#endif // _GLIBMM_USTRING_H
