#include "wrap.h"
#include "quark.h"
#include <glibmm/object.h>
#include <gobject/gobject.h>
#include <map>

#include <glibmmconfig.h>
GTKMM_USING_STD_MAP

namespace Glib
{

/********************************************************************/

typedef std::map<unsigned int, WrapNewFunction> Wrap_Table;
Wrap_Table* wrap_table = 0;

// Registers a new type, and its wrap_new() function, with wrap map
void wrap_register(const char* name, WrapNewFunction func)
{
  if (!wrap_table)
    wrap_table = new Wrap_Table();

  (*wrap_table)[string_hash(name)] = func;
}

// This is a factory function that converts any type to
// its C++ wrapper instance by looking up a wrap_new() function in a map.
Object* wrap_auto(GObject* o, bool take_copy)
{
  // screen out NULL
  if (!o)
    return 0;

  // Look up current C++ wrapper instance:
  Object* pCppObject = static_cast<Object*>(g_object_get_qdata(G_OBJECT(o), quark_));
  if(!pCppObject)  //If there's already a wrapper then return it:
  {
    // Generate a new C++ instance:
    if (!wrap_table)
    {
      g_warning("gtkmm wrapper table not yet filled.\n");
      return 0;
    }

    WrapNewFunction func_wrap_new = 0;
    GType type = G_OBJECT_TYPE(o);

    //Find the most-specialized wrap_new() for this GType.
    while (type)
    {
      func_wrap_new = (*wrap_table)[string_hash(g_type_name(type))];
      if(func_wrap_new)
      {
        //If there was a wrap_new() in the table then use it:
        pCppObject = (func_wrap_new)(o, take_copy);
        return pCppObject;
      }
      else
      {
        //Look for a wrap_new() for the parent GType instead.
        type = g_type_parent(type);
      }
    }
  }

  if(!pCppObject)
    g_warning("gtkmm failed to wrap type of %s.\n", g_type_name(G_OBJECT_TYPE(o)));

  return pCppObject;
}

// Silly little hash function to speed up lookups in table
unsigned int string_hash(const char* key)
{
  if (!key) return 0;
  const char *p = key;
  guint h = *p;

  if (h)
    for (p += 1; *p != '\0'; p++)
      h = (h << 5) - h + *p;

  return h;
}

Object* wrap(GObject* o, bool take_copy /* = false */)
{
  return static_cast<Object*>( wrap_auto((GObject*)o, take_copy) );
}

GObject* unwrap(Glib::Object* o)
{
  return (o ? o->gobj() : 0 );
}

} /* namespace Glib */
