#!/usr/bin/perl

use strict;

my $namespace_whole = "gtk";
my $path = "gtk--";
my $debug = 0;
my %objects = ();
my %namespaces = ();
my %nowrap = ();

while ($ARGV[0] =~ /^-/)
  {
   if ($ARGV[0] =~ /--namespace=(\S+)/) {$namespace_whole = "$1";}
   elsif ($ARGV[0] =~ /--debug/) {$debug = 1;}
   elsif ($ARGV[0] =~ /--path=(\S+)/) {$path = "$1";}
   else {print "Error: unknown option $ARGV[0]\n"; exit;}

   shift @ARGV;
  }

while ($ARGV[0])
  {
    if ($debug) {warn "Processing file : $ARGV[0]\n";}

    open FILE, $ARGV[0] or die "Couldn't open file $ARGV[0] : $!\n";
    my $file = $ARGV[0];
    $file =~ s/.*\/([^\/]+)$/$1/;
    $file =~ s/\.hg//;
    my $tmpnamespace = $namespace_whole;
    my $cppname = "";
    my $cname = "";
    while (<FILE>) 
      {
	      if (/CLASS_START\((\w+)\)/) #We need a new way to get the namespace.
	      {
          print "generate_wrap_init: namespace found: $1\n";
	        $tmpnamespace = $1;
	      }
	      elsif (/_CLASS_GOBJECT\s*\(/) #TODO: There is duplication of code here.
	      {
	        my $line = $_;
	        while ($line !~ /\)/ && ($_ = <FILE>))
	        {
		        $line .= $_;
	        }

	        $line =~ s/^.*_CLASS_GOBJECT\s*\(//;
	        $line =~ s/\s+//g;
	        ($cppname, $cname) = split(/,/, $line);

          $objects{$cppname} = $cname;
	        $namespaces{$cppname} = $tmpnamespace;
	      }
        elsif (/_CLASS_GTKOBJECT\s*\(/)
	      {
	        my $line = $_;
	        while ($line !~ /\)/ && ($_ = <FILE>))
	        {
		        $line .= $_;
	        }

	        $line =~ s/^.*_CLASS_GTKOBJECT\s*\(//;
	        $line =~ s/\s+//g;
	        ($cppname, $cname) = split(/,/, $line);

          if( ($cname ne "GtkTree") && ($cname ne "GtkTreeItem") && ($cname ne "GtkText") ) #TODO: Remove this hack eventually.
          {
	          $objects{$cppname} = $cname;
	          $namespaces{$cppname} = $tmpnamespace;
          }
	      }
        elsif (/_NO_WRAP_FUNCTION/)
        {
          $nowrap{$cppname} = 1;
        }
      }

    shift @ARGV;
    close(FILE);
  }

my $namespace_whole_lower = lc($namespace_whole);
print "#include <${namespace_whole_lower}mm/wrap_init.h>\n";
print "#include <glibmm/object.h>\n";

# Here we have to be subtle. The Gtk-- objects are easy, they all go
# into the Gtk namespace. But in Gnome--, some go into the Gnome
# namespace (most of them), and some into the Gtk one (because the
# corresponding widget is Gtk-prefixed, e.g. GtkTed, GtkDial, etc...

# First, the Gtk namespace

print "\n//Declarations of the *_Class::wrap_new() methods, instead of including all the private headers:\n\n";

my $i = 0;
foreach $i (sort keys %objects)
  {
    if( $nowrap{$i} ne 1 )
    {
      print "namespace $namespaces{$i} {class ${i}_Class { public: static Glib::Object* wrap_new(GObject* o, bool take_copy = false); }; }\n";
    }
  }

print "\n";
print "namespace $namespace_whole\n";
print "{\n";
print "\n";
print "void wrap_init()\n{\n";

# Generate namespace::wrap_init() body
#
foreach $i (sort keys %objects)
  {
    if( $nowrap{$i} ne 1 )
    {
      print "  Glib::wrap_register(\"$objects{$i}\", \&$namespaces{$i}::${i}_Class::wrap_new);\n";
    }
  }

print "} // wrap_init()\n";
print "\n";
print "} //${namespace_whole}\n";

